#' Eval readings of one PASTIS device
#' 
#' Eval readings of one PASTIS device
#' 
#' @description Data reads must have same reading intervals and readings times (made in the same minute). Output time in UTC.
#' 
#' @param obs Observations sensor readings (as output of \code{\link{readPASTIS}}).
#' @param ref Reference sensor readings (as output of \code{\link{readPASTIS}}).
#' 
#' @return Data.frame with PAI estimates at PASTIS native temporal resolution.
#' 
#' @export
#' 
#' @import lubridate
#' @import dplyr
#'
#' @author Benjamin Brede

require(dplyr)

# normal (wide) format (each sensor = 1 column)
evalPASTIS <- function(obs, ref) {
  
  # check that sensors are same for obs & ref
  if (!all(sort(getSensors(obs)) == sort(getSensors(ref))))
    stop('Sensor names do not match!')
  
  # merge tables
  df <- merge(ref, obs[,c(getSensors(obs), 'Temp', 'DTS')], by = 'DTS', suffixes = c('_ref', '_obs'))
  
  res <- do.call(cbind, lapply(getSensors(obs), function(sensor) {
    ob <- df[, paste0(sensor, '_obs')]
    re <- df[, paste0(sensor, '_ref')]
    
    r <- data.frame(ob / re)
    names(r) <- paste0(sensor, '_Pgap')
    r
  })) 
  
  cbind(select(df, DateTime, contains('_obs'), contains('_ref')), res)
}


# long format (all sensor readings in the same column, sensor name in another column)
evalPASTISlong <- function(obs, ref) {
  
  # check that sensors are same for obs & ref
  if (!all(sort(getSensors(obs)) == sort(getSensors(ref))))
    stop('Sensor names do not match!')
  
  # merge tables
  df <- merge(ref, obs[,c(getSensors(obs), 'Temp', 'DTS')], by = 'DTS', suffixes = c('_ref', '_obs'))
  
  do.call(rbind, lapply(getSensors(obs), function(sensor) {
    r <- data.frame(df[, paste0(sensor, '_obs')] / df[, paste0(sensor, '_ref')])
    names(r) <- 'Pgap'
    meta <- select(df, DateTime, DTS, Temp_obs, Temp_ref, paste0(sensor, '_obs'), paste0(sensor, '_ref'))
    names(meta) <- sub(paste0(sensor, '_ref'), 'Ref', names(meta))
    names(meta) <- sub(paste0(sensor, '_obs'), 'Obs', names(meta))
    cbind(meta, r, Sensor = sensor)
  })) 
}
