#' Match Obs and Ref readings
#' 
#' @description Data reads must have same reading intervals and readings times (made in the same minute). Output time in UTC.
#' 
#' @param obs Observations sensor readings (as output of \code{\link{readPASTIS}}).
#' @param ref Reference sensor readings (as output of \code{\link{readPASTIS}}).
#' 
#' @return Data.frame of matched readings (long format)
#' 
#' @export
#' 
#' @import lubridate
#' @import dplyr
#'
#' @author Benjamin Brede

require(lubridate)
require(dplyr)

# long format (all sensor readings in the same column, sensor name in another column)
matchPASTIS <- function(obs, ref) {
  
  # check that sensors are same for obs & ref
  if (!all(sort(getSensors(obs)) == sort(getSensors(ref))))
    stop('Sensor names do not match!')
  
  # DateTime in sec since 1970 for merge with other data later
  obs$DTS <- as.double(obs$DateTime)
  ref$DTS <- as.double(ref$DateTime)
  
  # merge tables
  df <- inner_join(ref, select(obs, getSensors(obs), Temp, DTS), by = 'DTS', suffix = c('_ref', '_obs'))
  
  do.call(rbind, lapply(getSensors(obs), function(sensor) {
    readings <- select(df, DateTime, DTS, Temp_obs, Temp_ref, paste0(sensor, '_obs'), paste0(sensor, '_ref'))
    # replace column names for any sensor with Ref/Obs
    names(readings) <- sub(paste0(sensor, '_ref'), 'Ref', names(readings))
    names(readings) <- sub(paste0(sensor, '_obs'), 'Obs', names(readings))
    readings$Sensor <- sensor
    readings
  })) 
}
