#' Read PASTIS files
#' 
#' @param file Filename of file to read.
#' @param sensors Labels referring to the 6 different sensors, typically referring to directions. Must be 6 unique labels.
#' @param tz Time zone of the system set up.
#' @param saturation Saturation level (DN).
#' @param add_all Add artificial sensor that is the sum of all other sensor readings.
#' 
#' @return Data.frame wit readings (wide format), meta info as attributes
#' 
#' @export
#' @import lubridate
#' 
#' @author Benjamin Brede

require(lubridate)

readPASTIS <- function(file, sensors = c('NW', 'SE', 'W', 'E', 'SW', 'NE'), tz = 'UTC', saturation = 4000, add_all = FALSE) {
  
  if (length(unique(sensors)) != 6)
    stop('Direction labels are not 6 unique labels!')
  
  df <- read.table(file, skip = 10, dec = '.', sep = ';', col.names = c('DateTime', 'ObsNo', sensors, 'Temp'))
  # DateTime, make sure it's in UTC
  df$DateTime <- with_tz(as.POSIXct(strptime(df$DateTime, '%d/%m/%Y - %T', tz)), 'UTC')
  # replace values > 4000 with NA (= saturation)
  df[sensors] <- lapply(df[sensors], function(x) replace(x, x >= saturation, NA))
  
  if (add_all) {
    df$All <- rowSums(as.matrix(df[sensors]))
    sensors <- c(sensors, 'All')
  }
  
  # add meta data as attributes
  header <- readLines(file, 8)
  attr(df, 'Sensors') <- sensors
  attr(df, 'Offset') <- as.numeric(sub('.*([0-9]+).*', '\\1', header[4]))
  attr(df, 'Device') <- as.numeric(sub('.*([0-9]+)', '\\1', header[6]))
  attr(df, 'Creation') <- as.POSIXct(strptime(sub('.*([0-9]{2}/[0-9]{2}/[0-9]{4}).*([0-9]{2}:[0-9]{2}:[0-9]{2}).*', '\\1 \\2', header[3]), '%d/%m/%Y %H:%M:%S', tz))
  attr(df, 'Interval') <- as.numeric(sub('.*([0-9]+).*', '\\1', header[8]))
  
  df
}

getSensors <- function(pastis) {
  attr(pastis, 'Sensors')
}

getOffset <- function(pastis) {
  attr(pastis, 'Offset')
}

getDevice <- function(pastis) {
  attr(pastis, 'Device')
}

getCreation <- function(pastis) {
  attr(pastis, 'Creation')
}

getInterval <- function(pastis) {
  attr(pastis, 'Interval')
}

